/////////////////////////////////////////////////////////////////////
// Context.h                                                       //
// ver 1.0                                                         //
// Yuan Liu, Demo program for CSE 776, Fall 2018                   //
/////////////////////////////////////////////////////////////////////
/*
*  File Operations:
*  -------------------
*  - This file defines a simple demo of the state pattern.
*    you can choose an operation among RUN, STOP, CLOSE, 
*    and OPEN, and use an instance of Context to apply it.
*  - This calss is also responsible for the state transition.
*
*
*  Maintenance History:
*  --------------------
*  ver 1.0 : 18 Sep 2018
*  - first release
*/
#pragma once
#include "State.h"
#include "OpenState.h"
#include "CloseState.h"
#include "RunState.h"
#include "StopState.h"
#include <map>
#include <string>
#include <iostream>

using std::map;
using std::string;
using std::cout;
using std::endl;

class Context {
public:
    Context();
    ~Context();
    void open();                        // open operation
    void close();                       // close operation
    void run();                         // run operation
    void stop();                        // stop operation

    static OpenState *openState;        // an instance of OPEN state
    static CloseState *closeState;      // an instance of CLOSE state
    static RunState *runState;          // an instance of RUN state
    static StopState *stopState;        // an instance of STOP state

private:
    State *state;                       // current state instance
    string currentState;                // record current state with a string
    map<string, map<string, bool> > stateMap;   // define the valid state transition
};